@@ -15,9 +15,11 @@ except ImportError:
 from shlex import split as shlex_split
 
 from .action_processor import get_actions, list_actions
+from .aliases import AliasManager
 from .api import get_soco_object, run_command
 from .cmd_parser import CLIParser
 from .utils import (
+    RewindableList,
     get_readline_history,
     get_speaker,
     local_speaker_list,
@@ -28,6 +30,11 @@ from .utils import (
 from .wait_actions import process_wait
 
 
+# Alias Manager
+am = AliasManager()
+am.load_aliases()
+
+
 def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
 
     # Is the speaker name set on the command line?
@@ -44,9 +51,7 @@ def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
     print("\nEntering SoCo-CLI interactive shell.")
     print("Type 'help' for available shell commands.\n")
     if not RL:
-        print(
-            "Note: Command history and autocompletion not currently available on Windows.\n"
-        )
+        print("Note: Autocompletion not currently available on Windows.\n")
 
     if RL:
         _set_actions_and_commands_list(use_local_speaker_list=use_local_speaker_list)
@@ -72,12 +77,23 @@ def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
         cli_parser.parse(shlex_split(command_line))
 
         # Loop through action sequences
-        for command in cli_parser.get_sequences():
+        command_sequences = RewindableList(cli_parser.get_sequences())
+        logging.info("Command sequences = {}".format(command_sequences))
+
+        # The command_sequence list can change, so we use pop() until the
+        # list is exhausted
+        while True:
+            try:
+                command = command_sequences.pop_next()
+                logging.info("Current command = '{}'".format(command))
+            except IndexError:
+                break
 
             command_lower = command[0].lower()
 
             if command[0] == "0":
                 # Unset the active speaker
+                logging.info("Unset active speaker")
                 speaker_name = None
                 speaker = None
                 continue
@@ -100,6 +116,7 @@ def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
                 continue
 
             if command_lower in ["wait", "wait_until", "wait_for"]:
+                logging.info("Processing a 'wait' action")
                 process_wait(command)
                 continue
 
@@ -110,10 +127,14 @@ def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
                     _get_speaker_names(use_local_speaker_list=use_local_speaker_list)
                 )
                 if 1 <= speaker_number <= limit:
+                    logging.info(
+                        "Setting to active speaker no. {} ".format(speaker_number)
+                    )
                     speaker_name = _get_speaker_names(
                         use_local_speaker_list=use_local_speaker_list
                     )[speaker_number - 1]
                     speaker = get_speaker(speaker_name, use_local_speaker_list)
+                    logging.info("{} : {}".format(speaker, speaker_name))
                 else:
                     print(
                         "Error: Speaker number is out of range (0 to {})".format(limit)
@@ -133,6 +154,7 @@ def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
                 if use_local_speaker_list:
                     print("Using cached speaker list: no rescan performed")
                 else:
+                    logging.info("Full network rescan")
                     speaker_cache().scan(reset=True)
                     _print_speaker_list(use_local_speaker_list=use_local_speaker_list)
                     _set_actions_and_commands_list(
@@ -140,8 +162,65 @@ def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
                     )
                 continue
 
+            if command_lower == "alias":
+                if len(command) == 1:
+                    am.print_aliases()
+                    continue
+                # Remove 'alias'
+                command.pop(0)
+                alias_name = command.pop(0)
+                if len(command) == 0:
+                    if am.create_alias(alias_name, None):
+                        print("Alias '{}' removed".format(alias_name))
+                    else:
+                        print("Alias '{}' not found".format(alias_name))
+                else:
+                    # Have to collect the remaining sequences: they're all
+                    # part of the alias. Reconstruction required.
+                    actions = [" ".join(command)]
+                    while True:
+                        try:
+                            command = command_sequences.pop_next()
+                            actions.append(" ".join(command))
+                        except IndexError:
+                            break
+                    action = " : ".join(actions)
+                    _, new = am.create_alias(alias_name, action)
+                    if new:
+                        print("Alias '{}' created".format(alias_name))
+                    else:
+                        print("Alias '{}' overwritten".format(alias_name))
+                am.save_aliases()
+                _set_actions_and_commands_list(
+                    use_local_speaker_list=use_local_speaker_list
+                )
+                continue
+
             # Command processing
             try:
+
+                # Replace the command sequence with the contents of an alias
+                # This is tricky for multiple sequences
+                if command[0] in am.alias_names():
+                    action = am.action(command[0])
+                    actions = shlex_split(action)
+                    cli_parser.parse(actions)
+                    new_sequences = cli_parser.get_sequences()
+                    logging.info("Sequences in the alias: {}".format(new_sequences))
+                    command = new_sequences.pop(0)
+                    # Multiple sequences?
+                    if len(new_sequences):
+                        # Insert the additional sequences in the list
+                        index = command_sequences.index()
+                        for sequence in new_sequences:
+                            logging.info(
+                                "Inserting new sequence {} at {}".format(
+                                    sequence, index
+                                )
+                            )
+                            command_sequences.insert(index, sequence)
+                            index += 1
+
                 args = command
 
                 # Setting a speaker to operate on?
@@ -156,6 +235,9 @@ def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
                                 "Error: Speaker '{}' not found".format(new_speaker_name)
                             )
                         else:
+                            logging.info(
+                                "Set new active speaker: {}'".format(speaker_name)
+                            )
                             speaker_name = new_speaker_name
                             speaker = new_speaker
                         continue
@@ -164,12 +246,16 @@ def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
                     continue
 
                 if not speaker_name:
+                    logging.info(
+                        "Treating first parameter '{}' as speaker name".format(args[0])
+                    )
                     speaker = get_speaker(args.pop(0), use_local_speaker_list)
                     if not speaker:
                         print("Error: Speaker not found")
                         continue
 
                 action = args.pop(0).lower()
+                logging.info("Action = '{}'; args = '{}'".format(action, args))
                 exit_code, output, error_msg = run_command(
                     speaker,
                     action,
@@ -191,7 +277,15 @@ def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
                 print("Error: Invalid command")
 
 
-COMMANDS = ["actions", "exit", "help", "rescan", "set ", "speakers"]
+COMMANDS = [
+    "actions",
+    "alias ",
+    "exit",
+    "help",
+    "rescan",
+    "set ",
+    "speakers",
+]
 
 
 def _completer(text, context):
@@ -213,12 +307,17 @@ ACTIONS_LIST = []
 
 
 def _set_actions_and_commands_list(use_local_speaker_list=False):
+    logging.info("Rebuilding commands/action list")
     global ACTIONS_LIST
-    ACTIONS_LIST = [
-        action + " "
-        for action in get_actions()
-        + _get_speaker_names(use_local_speaker_list=use_local_speaker_list)
-    ] + COMMANDS
+    ACTIONS_LIST = (
+        [
+            action + " "
+            for action in get_actions()
+            + _get_speaker_names(use_local_speaker_list=use_local_speaker_list)
+        ]
+        + COMMANDS
+        + am.alias_names()
+    )
 
 
 def _get_actions_and_commands():
@@ -237,6 +336,13 @@ This is SoCo-CLI interactive mode. Interactive commands are as follows:
                     in the list, just type '4'.
                     '0' will unset the active speaker.
     'actions'   :   Show the complete list of SoCo-CLI actions.
+    'alias'     :   Add an alias: alias <alias_name> <actions>
+                    Remove an alias: alias <alias_name>
+                    Update an alias by creating a new alias with the same name.
+                    Using 'alias' without parameters shows the current list of
+                    aliases.
+                    Aliases override existing actions and can contain
+                    sequences of actions.
     'exit'      :   Exit the shell.
     'help'      :   Show this help message (available shell commands).
     'rescan'    :   If your speaker doesn't appear in the 'speakers' list,
@@ -251,10 +357,11 @@ This is SoCo-CLI interactive mode. Interactive commands are as follows:
     
     The command syntax is the same as when using 'sonos' from the command line.
     If a speaker been set, the speaker name is omitted from the command.
+
+    Use the arrow keys for command history and command editing.
     
-    [Not Available on Windows]
-    Use the TAB key for autocompletion of shell commands, SoCo-CLI actions, and
-    speaker names. Use the arrow keys for command history and command editing.
+    [Not Available on Windows] Use the TAB key for autocompletion of shell
+    commands, SoCo-CLI actions, aliases, and speaker names.
 """
 
 
@@ -277,9 +384,11 @@ def _print_speaker_list(use_local_speaker_list=False):
 
 def _save_readline_history():
     if RL:
+        logging.info("Saving shell history")
         save_readline_history()
 
 
 def _get_readline_history():
     if RL:
+        logging.info("Reading shell history")
         get_readline_history()
